  /***************************************************************
 ******************************************************************
****	    Gaussian Elimination with partial pivoting.		****
****	This file contains the solution routine SGESL		****
 ******************************************************************
  ****************************************************************/

#include "ge.h"
static char SGESLSid[] = "@(#)sgesl.c	1.3  4/8/88";

void sgesl( a, ipvt, b, job )
struct FULL *a;
int 	    *ipvt, job;
float	    b[];
/*
    PURPOSE
	SGESL solves the real system
	a * x = b  or  trans(a) * x = b
	using the factors computed by SGECO or SGEFA.

    INPUT
	a	A pointer to the FULL matrix structure containing the factored
		matrix.  See the definition of FULL in ge.h.
	ipvt    The pivot vector (of length a->cd) from SGECO or SGEFA.
	b       The right hand side vector (of length a->cd).
	job     = 0         to solve  a*x = b ,
		= nonzero   to solve  trans(a)*x = b  where
			    trans(a)  is the transpose.

    OUTPUT
	b       The solution vector x. 

    REMARKS
	Error condition:
	A division by zero will occur if the input factor contains a
	zero on the diagonal.  Technically this indicates singularity
	but it is often caused by improper arguments or improper
	setting of lda .  It will not occur if the subroutines are
	called correctly and if sgeco has set rcond .gt. 0.0
	or sgefa has set info .eq. 0 .
*/
{
    register float t, *mik;
    float	*akk;
    register int i, k;
    int	l, n, nm1;
    
    n   = a->cd;
    nm1 = n - 1;
    
    /* job = 0 , solve  A * x = b.  */
    if( job == 0 ) {

	/* Forward elimination. Solve L*y = b. */
	for( k=0; k<nm1; k++ ) {
	    akk = a->pd[k] + k;		/* akk points to a(k,k). */
	    
	    /* Interchange b[k] and b[l] if necessary. */
	    l = ipvt[k];
	    t = b[l];
	    if( l != k ) {
		b[l] = b[k];
		b[k] = t;
	    }
	    for( i=k+1, mik=a->pd[k]; i<n; i++ )
	      b[i] += t*mik[i];
	}
	
	/* Back substitution.  Solve  U*x = y. */
	for( k=nm1; k>=0; k-- ) {
	    register float *uik = a->pd[k];
	    akk = uik + k;
	    b[k] /= (*akk);
	    for( i=0; i<k; i++ )
	      b[i] -= uik[i]*b[k];
	}
	return;
    }
    
    /* job = nonzero.  Solve  trans(A) * x = b. */
    /* First solve trans(U)*y = b. */
    for( k=0; k<n; k++ ) {
	register float *uik = a->pd[k];
	akk = uik + k;
	t = 0.0;
	for( i=0; i<k; i++ )
	  t += uik[i]*b[i];
	b[k] = (b[k] - t) / (*akk);
    }
    
    /* b now contains y. */
    /* Solve trans(L)*x = y. */
    for( k=n-2; k>=0; k-- ) {
	mik = a->pd[k];
	t = 0.0;
	for( i=k+1; i<n; i++ )
	  t += mik[i]*b[i];
	b[k] += t;
	
	/* Interchange b(k) and b(ipvt(k)) if necessary. */
	l    = ipvt[k];
	if( l == k ) continue; 
	t    = b[l];
	b[l] = b[k];
	b[k] = t;
    }
    return;
}
